<?php
/**
 * A database wrapper for region informations
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class RegionInformationDatabaseWrapper extends BaseDatabaseWrapper implements RegionInformationWrapper, Registerable {
	// Constants for database table names
	const DB_TABLE_REGION_INFORMATION = 'region_data';

	// Constants for database column names
	const DB_COLUMN_REGION_ID          = 'region_id';
	const DB_COLUMN_REGION_NAME        = 'region_name';
	const DB_COLUMN_REGION_USER_ID     = 'region_user_id';

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this database wrapper by a provided user class
	 *
	 * @return	$wrapperInstance	An instance of the created wrapper class
	 */
	public static final function createRegionInformationDatabaseWrapper () {
		// Get a new instance
		$wrapperInstance = new RegionInformationDatabaseWrapper();

		// Set (primary!) table name
		$wrapperInstance->setTableName(self::DB_TABLE_REGION_INFORMATION);

		// Return the instance
		return $wrapperInstance;
	}

	/**
	 * Removes non-public data from given array.
	 *
	 * @param	$data	An array with possible non-public data that needs to be removed.
	 * @return	$data	A cleaned up array with only public data.
	 */
	public function removeNonPublicDataFromArray(array $data) {
		// Currently call only inner method
		/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('REGION-WRAPPER[' . __METHOD__ . ':' . __LINE__ . ']: Calling parent::removeNonPublicDataFromArray(data) ...');
		$data = parent::removeNonPublicDataFromArray($data);

		// Return cleaned data
		return $data;
	}

	/**
	 * Checks whether the user has already founded a region
	 *
	 * @return	$hasFounded		Whether the user has already founded a region
	 */
	public function ifUserHasCreatedRegion () {
		// Get user instance
		$userInstance = Registry::getRegistry()->getInstance('user');

		// Now get a search criteria instance
		$searchInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Search for user's cities
		$searchInstance->addCriteria(RegionInformationDatabaseWrapper::DB_COLUMN_REGION_USER_ID, $userInstance->getUserId());

		// Get a result back
		$resultInstance = $this->doSelectByCriteria($searchInstance);

		// Get region manager instance
		$managerInstance = ManagerFactory::createManagerByType('region');

		// Set result instance
		$managerInstance->setResultInstance($resultInstance);

		// Has it been founded?
		$hasFounded = $resultInstance->valid();

		// Return result
		return $hasFounded;
	}

	/**
	 * Checks whether the given region name is taken
	 *
	 * @param	$regionName		Name of region
	 * @return	$isTaken		Whether the given region name is taken
	 */
	public function ifRegionExists ($regionName) {
		// Now get a search criteria instance
		$searchInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Search for the region number one which is hard-coded the default
		$searchInstance->addCriteria(RegionInformationDatabaseWrapper::DB_COLUMN_REGION_NAME, $regionName);
		$searchInstance->setLimit(1);

		// Get a result back
		$resultInstance = $this->doSelectByCriteria($searchInstance);

		// Check it
		$isTaken = $resultInstance->next();
		//* NOISY-DEBUG: */ $this->debugOutput('[' . __METHOD__ . ':' . __LINE__ . '] isTaken[' . gettype($isTaken) . ']=' . intval($isTaken));

		// Get manger instance
		$managerInstance = ManagerFactory::createManagerByType('region');

		// Set result instance
		$managerInstance->setResultInstance($resultInstance);

		// Return result
		return $isTaken;
	}

	/**
	 * Creates a region by given name
	 *
	 * @param	$regionName		Name of region
	 * @return	void
	 */
	public function createRegionByName ($regionName) {
		// Pre-check name
		assert(!$this->ifRegionExists($regionName));

		// Get user instance
		$userInstance = Registry::getRegistry()->getInstance('user');

		// Get a dataset instance
		$dataSetInstance = ObjectFactory::createObjectByConfiguredName('dataset_criteria_class', array(self::DB_TABLE_REGION_INFORMATION));

		// Set the primary key
		$dataSetInstance->setUniqueKey(self::DB_COLUMN_REGION_ID);

		// Add region name and assign user id
		$dataSetInstance->addCriteria(self::DB_COLUMN_REGION_ID     , ($this->countTotalRows() + 1));
		$dataSetInstance->addCriteria(self::DB_COLUMN_REGION_NAME   , $regionName);
		$dataSetInstance->addCriteria(self::DB_COLUMN_REGION_USER_ID, $userInstance->getUserId());

		// "Insert" this dataset instance completely into the database
		$this->queryInsertDataSet($dataSetInstance);

		// Post-check name
		assert($this->ifRegionExists($regionName));
	}
}

// [EOF]
?>
